package com.ruoyi.system.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.JSONWriter;
import com.ruoyi.common.core.constant.Constants;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.domain.StrFile;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.core.utils.ServletUtils;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.core.utils.bean.BeanUtils;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.security.service.TokenService;
import com.ruoyi.system.api.RemoteFileService;
import com.ruoyi.system.api.domain.SysFile;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.system.domain.SnServer;
import com.ruoyi.system.domain.SnServerLog;
import com.ruoyi.system.domain.SnServerTask;
import com.ruoyi.system.domain.SnServerTemplate;
import com.ruoyi.system.domain.SnSslLog;
import com.ruoyi.system.domain.dto.SnServerLogDto;
import com.ruoyi.system.enums.TaskTypeEnum;
import com.ruoyi.system.kernel.ServerLogKernel;
import com.ruoyi.system.kernel.SslCountLimit;
import com.ruoyi.system.mapper.SnServerLogMapper;
import com.ruoyi.system.mapper.SnServerMapper;
import com.ruoyi.system.mapper.SnServerTaskMapper;
import com.ruoyi.system.mapper.SnServerTemplateMapper;
import com.ruoyi.system.mapper.SnSslLogMapper;
import com.ruoyi.system.mq.ServerTaskSender;
import com.ruoyi.system.service.ISnServerService;
import com.ruoyi.system.service.ISnServerTaskService;
import com.ruoyi.system.utils.SSLExpirationChecker;
import com.ruoyi.system.utils.dbsync.runner.Syncer;
import com.ruoyi.system.utils.jsch.AnsiStripper;
import com.ruoyi.system.utils.jsch.ServiceShell;
import java.util.Comparator;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

@Service
public class SnServerTaskServiceImpl implements ISnServerTaskService {
   private static final Logger log = LoggerFactory.getLogger(ServerTaskSender.class);
   @Autowired
   private SnServerTaskMapper snServerTaskMapper;
   @Autowired
   private TokenService tokenService;
   @Autowired
   private SnServerLogMapper snServerLogMapper;
   @Autowired
   private SnServerMapper snServerMapper;
   @Autowired
   private SnServerTemplateMapper snServerTemplateMapper;
   @Autowired
   private SnSslLogMapper snSslLogMapper;
   @Autowired
   private ServerLogKernel serverLogKernel;
   @Autowired
   private RemoteFileService remoteFileService;
   @Autowired
   private SslCountLimit sslCountLimit;
   @Autowired
   private ISnServerService snServerService;
   @Value("${dbSync.url}")
   public String dbSyncUrl;
   @Value("${dbSync.password}")
   public String dbSyncPsd;

   public AjaxResult queryLastStatus(SnServerTask param) {
      SnServerTask task = null;
      if (param.getServerId() != null) {
         task = this.snServerTaskMapper.queryLastTask(param.getServerId());
      } else if (param.getTaskId() != null) {
         task = this.snServerTaskMapper.selectSnServerTaskByTaskId(param.getTaskId());
      }

      if (task == null) {
         return AjaxResult.error("未查询到数据");
      } else {
         SnServerLog log = new SnServerLog();
         log.setTaskId(task.getTaskId());
         List<SnServerLog> logs = this.snServerLogMapper.selectSnServerLogList(log);
         return AjaxResult.success((new JSONObject()).fluentPut("status", task.getStatus()).fluentPut("result", task.getResult()).fluentPut("logs", BeanUtils.copyList(logs, SnServerLogDto.class)));
      }
   }

   public SnServerTask selectSnServerTaskByTaskId(Long taskId) {
      SnServerTask obj = this.snServerTaskMapper.selectSnServerTaskByTaskId(taskId);
      if (obj != null && !obj.getIsDelete().equals(1)) {
         LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
         if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super")) {
            if (loginUser.getRoles().contains("dealer")) {
               return obj.getDealerId().equals(loginUser.getUserid()) ? obj : null;
            } else if (loginUser.getRoles().contains("agent")) {
               return obj.getAgentId().equals(loginUser.getUserid()) ? obj : null;
            } else {
               return obj.getCreateId().equals(loginUser.getUserid()) ? obj : null;
            }
         } else {
            return obj;
         }
      } else {
         return null;
      }
   }

   public List<SnServerTask> selectSnServerTaskList(SnServerTask obj) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      if (loginUser.getRoles().contains("dealer")) {
         obj.setDealerId(loginUser.getUserid());
      }

      if (loginUser.getRoles().contains("agent")) {
         obj.setAgentId(loginUser.getUserid());
      }

      if (loginUser.getRoles().contains("common")) {
         obj.setCreateId(loginUser.getUserid());
      }

      obj.setIsDelete(0);
      return this.snServerTaskMapper.selectSnServerTaskList(obj);
   }

   public AjaxResult insertSnServerTask(SnServerTask snServerTask) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super") && !loginUser.getSysUser().getUserType().equals("common")) {
         return AjaxResult.error("暂无权限");
      } else {
         snServerTask.setCreateId(loginUser.getUserid());
         snServerTask.setCreateName(loginUser.getUsername());
         snServerTask.setAgentId(Long.parseLong(loginUser.getSysUser().getCreateBy()));
         snServerTask.setAgentName(loginUser.getSysUser().getCreateName());
         snServerTask.setIsDelete(0);
         snServerTask.setCreateTime(DateUtils.getNowDate());
         snServerTask.setUpdateTime(DateUtils.getNowDate());
         return AjaxResult.success(this.snServerTaskMapper.insertSnServerTask(snServerTask));
      }
   }

   public int updateSnServerTask(SnServerTask snServerTask) {
      if (this.selectSnServerTaskByTaskId(snServerTask.getTaskId()) == null) {
         return 0;
      } else {
         LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
         snServerTask.setUpdateId(loginUser.getUserid());
         snServerTask.setUpdateName(loginUser.getUsername());
         snServerTask.setUpdateTime(DateUtils.getNowDate());
         return this.snServerTaskMapper.updateSnServerTask(snServerTask);
      }
   }

   public int deleteSnServerTaskByTaskIds(Long[] taskIds) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      Long createId = null;
      if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super")) {
         createId = loginUser.getUserid();
         return this.snServerTaskMapper.deleteSnServerTaskByTaskIdsV2(taskIds, createId);
      } else {
         return this.snServerTaskMapper.deleteSnServerTaskByTaskIds(taskIds);
      }
   }

   public int deleteSnServerTaskByTaskId(Long taskId) {
      return this.snServerTaskMapper.deleteSnServerTaskByTaskId(taskId);
   }

   public void handleDoTask(Long taskId) {
      SnServerTask task = this.snServerTaskMapper.selectSnServerTaskByTaskId(taskId);
      if (task != null && !Objects.equals(task.getIsDelete(), 1)) {
         SnServerLog param = new SnServerLog();
         param.setTaskId(taskId);
         List<SnServerLog> logs = this.snServerLogMapper.selectSnServerLogList(param);
         if (logs.isEmpty()) {
            SnServerTask up = new SnServerTask();
            up.setTaskId(taskId);
            up.setStatus(3);
            up.setResult("当前任务没有需要执行的命令");
            this.snServerTaskMapper.updateSnServerTask(up);
         } else {
            logs.sort(Comparator.comparingLong(SnServerLog::getLogId));
            SnServer server = this.snServerMapper.selectSnServerByServerId(task.getServerId());
            if (!Objects.equals(server.getIsDelete(), 1)) {
               ServiceShell shell = new ServiceShell(task.getServerId(), this.serverLogKernel);

               try {
                  if (TaskTypeEnum.isConnect(task.getType())) {
                     shell.connect(server.getIp(), server.getUserName(), server.getDePassword(), server.getPort());
                  }

                  SnServerTask up = new SnServerTask();
                  up.setTaskId(task.getTaskId());
                  up.setStatus(1);
                  up.setUpdateTime(DateUtils.getNowDate());
                  this.snServerTaskMapper.updateSnServerTask(up);
                  Iterator var23 = logs.iterator();

                  while(var23.hasNext()) {
                     SnServerLog log = (SnServerLog)var23.next();
                     long time = System.currentTimeMillis();
                     SnServerLog upObj = new SnServerLog();
                     upObj.setLogId(log.getLogId());
                     upObj.setStatus(1);
                     upObj.setUpdateTime(DateUtils.getNowDate());
                     this.snServerLogMapper.updateSnServerLog(upObj);
                     String returnLog;
                     String command;
                     if (log.getOptName().equals("基础表结构同步")) {
                        command = server.getIp() + ":3306";
                        upObj.setCommand(this.dbSyncUrl + "====>" + command);
                        returnLog = Syncer.doSync(this.dbSyncUrl, this.dbSyncPsd, command, server.getDeDbPsd(), server.getVersion());
                        if (returnLog.contains("数据库连接失败")) {
                           upObj.setLogText(returnLog);
                           upObj.setStatus(3);
                           upObj.setTime(System.currentTimeMillis() - time);
                           upObj.setUpdateTime(DateUtils.getNowDate());
                           this.snServerLogMapper.updateSnServerLog(upObj);
                           if (task.getTaskName().equals("更新数据库表结构")) {
                              up.setStatus(3);
                              up.setResult("客户端数据库连接失败，请检查3306端口是否设置白名单");
                              up.setUpdateTime(DateUtils.getNowDate());
                              this.snServerTaskMapper.updateSnServerTask(up);
                              this.dealUpdateServer(task);
                              return;
                           }
                           break;
                        }
                     } else if (log.getOptName().equals("更新爬虫UA库版本")) {
                        Boolean updateUa = Objects.equals(task.getType(), TaskTypeEnum.uaUpdate.getType());
                        returnLog = this.snServerService.syncUa(server, updateUa);
                        if (StringUtils.isNotEmpty(returnLog)) {
                           upObj.setLogText(returnLog);
                           upObj.setStatus(3);
                           upObj.setTime(System.currentTimeMillis() - time);
                           upObj.setUpdateTime(DateUtils.getNowDate());
                           this.snServerLogMapper.updateSnServerLog(upObj);
                           if (updateUa) {
                              up.setStatus(3);
                              up.setResult("执行失败");
                              up.setUpdateTime(DateUtils.getNowDate());
                              this.snServerTaskMapper.updateSnServerTask(up);
                              this.dealUpdateServer(task);
                              return;
                           }
                           break;
                        }
                     } else if (log.getOptName().equals("缓存证书文件")) {
                        this.cacheEncrypt(server, task, log.getCommand());
                        returnLog = "缓存成功";
                     } else {
                        command = log.getCommand();
                        if (command.startsWith("exec ")) {
                           returnLog = shell.executeCommand(command.substring(5));
                        } else {
                           Integer timoutMinute = null;
                           if (Objects.equals(log.getOptName(), "安装服务器基础环境")) {
                              timoutMinute = 10;
                           }

                           returnLog = shell.sendCommand(command, timoutMinute);
                           returnLog = AnsiStripper.stripAnsi(returnLog);
                        }

                        if (log.getOptName().equals("插入商户许可证")) {
                           returnLog = this.retryInsertLicense(command, shell, returnLog, 1);
                        }
                     }

                     if (StringUtils.isNotEmpty(returnLog)) {
                        StrFile strFile = new StrFile();
                        strFile.setContent(returnLog);
                        strFile.setFileName("logs_" + log.getLogId() + ".txt");
                        strFile.setContentType("text/plain");
                        R<SysFile> r = this.remoteFileService.uploadStrToMinioV2(JSONObject.toJSONString(strFile, new JSONWriter.Feature[0]));
                        upObj.setLogText(r.getData() != null ? ((SysFile)r.getData()).getUrl() : "");
                     }

                     if (this.checkError(task, upObj, time, returnLog)) {
                        return;
                     }

                     upObj.setStatus(2);
                     upObj.setTime(System.currentTimeMillis() - time);
                     upObj.setUpdateTime(DateUtils.getNowDate());
                     this.snServerLogMapper.updateSnServerLog(upObj);
                  }

                  up.setStatus(2);
                  up.setResult("执行完成");
                  up.setUpdateTime(DateUtils.getNowDate());
                  this.snServerTaskMapper.updateSnServerTask(up);
                  SnServer serUp;
                  if (Objects.equals(task.getType(), TaskTypeEnum.installBase.getType())) {
                     serUp = new SnServer();
                     serUp.setServerId(task.getServerId());
                     serUp.setStatus(2);
                     serUp.setUpdateTime(DateUtils.getNowDate());
                     this.snServerMapper.updateSnServer(serUp);
                  }

                  if (Objects.equals(task.getType(), TaskTypeEnum.installClient.getType())) {
                     SnServerTemplate seTempUp = new SnServerTemplate();
                     seTempUp.setId(task.getServerTemplateId());
                     seTempUp.setStatus(2);
                     seTempUp.setUpdateTime(DateUtils.getNowDate());
                     this.snServerTemplateMapper.updateSnServerTemplate(seTempUp);
                  }

                  if (task.getProcess() != null) {
                     serUp = new SnServer();
                     serUp.setServerId(task.getServerId());
                     serUp.setProcess(task.getProcess());
                     serUp.setTaskId(0L);
                     serUp.setUpdateTime(DateUtils.getNowDate());
                     this.snServerMapper.updateSnServer(serUp);
                  }
               } catch (Exception var19) {
                  Exception e = var19;
                  SnServerTask up = new SnServerTask();
                  up.setTaskId(task.getTaskId());
                  up.setStatus(3);
                  up.setResult("任务执行异常：" + e.getMessage());
                  up.setUpdateTime(DateUtils.getNowDate());
                  this.snServerTaskMapper.updateSnServerTask(up);
                  this.dealUpdateServer(task);
               } finally {
                  shell.disconnect();
               }

            }
         }
      } else {
         SnServerTaskServiceImpl.log.info("未查询到任务" + taskId);
      }
   }

   private Boolean checkError(SnServerTask task, SnServerLog upObj, long startTime, String returnLog) {
      String msg = "";
      if (StringUtils.isEmpty(returnLog)) {
         return Boolean.FALSE;
      } else {
         if (Objects.equals(task.getType(), TaskTypeEnum.deploySSL.getType())) {
            if (returnLog.contains("domains in the last 168 hours")) {
               msg = "部署次数超限";
            } else if (returnLog.contains("DNS A/AAAA record(s) for that domain")) {
               msg = "域名未解析到当前IP";
            } else if (returnLog.contains("Which services should be restarted")) {
               msg = "安装过程中遇到需要选择服务重启，无法自动选择，请重新安装证书";
            } else if (returnLog.contains("Certbot failed to authenticate some domains")) {
               msg = "未能对域名进行身份验证，请确保域名正确解析到IP";
            }
         }

         if (returnLog.contains("空闲弹出")) {
            msg = "执行异常，超时关闭通道";
         }

         if (StringUtils.isNotEmpty(msg)) {
            upObj.setStatus(3);
            upObj.setTime(System.currentTimeMillis() - startTime);
            upObj.setUpdateTime(DateUtils.getNowDate());
            this.snServerLogMapper.updateSnServerLog(upObj);
            SnServerTask up = new SnServerTask();
            up.setTaskId(task.getTaskId());
            up.setStatus(3);
            up.setResult(msg);
            up.setUpdateTime(DateUtils.getNowDate());
            this.snServerTaskMapper.updateSnServerTask(up);
            return Boolean.TRUE;
         } else {
            return Boolean.FALSE;
         }
      }
   }

   private void dealUpdateServer(SnServerTask task) {
      SnServer serUp = null;
      if (Objects.equals(task.getType(), TaskTypeEnum.installBase.getType())) {
         serUp = new SnServer();
         serUp.setServerId(task.getServerId());
         serUp.setStatus(3);
         serUp.setDomain("");
         serUp.setVersion("");
         serUp.setBackVersion("");
         serUp.setUiVersion("");
         serUp.setDbPsd("");
      }

      if (Objects.equals(task.getType(), TaskTypeEnum.updateBaseUI.getType())) {
         serUp = new SnServer();
         serUp.setServerId(task.getServerId());
         serUp.setVersion(task.getClientVersion());
         serUp.setUiVersion(task.getBizVersion());
      }

      if (Objects.equals(task.getType(), TaskTypeEnum.updateClient.getType())) {
         SnServerTemplate seTempUp = new SnServerTemplate();
         seTempUp.setId(task.getServerTemplateId());
         seTempUp.setVersion(task.getBizVersion());
         this.snServerTemplateMapper.updateSnServerTemplate(seTempUp);
      }

      if (Objects.equals(task.getType(), TaskTypeEnum.updateBack.getType())) {
         serUp = new SnServer();
         serUp.setServerId(task.getServerId());
         serUp.setVersion(task.getClientVersion());
         serUp.setBackVersion(task.getBizVersion());
      }

      if (Objects.equals(task.getType(), TaskTypeEnum.uaUpdate.getType())) {
         serUp = new SnServer();
         serUp.setServerId(task.getServerId());
         serUp.setUaVersion(task.getBizVersion());
      }

      if (serUp != null) {
         this.snServerMapper.updateSnServer(serUp);
      }

   }

   private String retryInsertLicense(String command, ServiceShell shell, String returnLog, int num) {
      try {
         if (returnLog.contains("Can't connect to local MySQL")) {
            if (num > 5) {
               return "重试5次未能成功连接数据库";
            }

            Thread.sleep(2000L);
            if (command.startsWith("exec ")) {
               returnLog = shell.executeCommand(command.substring(5));
            } else {
               returnLog = shell.sendCommand(command);
               returnLog = AnsiStripper.stripAnsi(returnLog);
            }

            ++num;
            this.retryInsertLicense(command, shell, returnLog, num);
         }
      } catch (Exception var6) {
      }

      return returnLog;
   }

   @Async
   public void cacheEncrypt(SnServer server, SnServerTask task, String command) {
      ServiceShell shell = new ServiceShell(server.getServerId(), this.serverLogKernel);
      SnSslLog sslLog = new SnSslLog();
      sslLog.setExpireTime(DateUtils.currentTimeAdd(90));

      label87: {
         try {
            String[] arr = command.split(" && ");
            String pemCommand = arr[0];
            String keyCommand = arr[1];
            String domain = pemCommand.split("/")[4];
            sslLog.setDomain(domain);
            this.sslCountLimit.addCount(domain, server.getIp());
            shell.connect(server.getIp(), server.getUserName(), server.getDePassword(), server.getPort());
            String pemStr = shell.executeCommand(pemCommand);
            sslLog.setExpireTime(SSLExpirationChecker.getCertificateExDate(pemStr));
            String keyStr = shell.executeCommand(keyCommand);
            StrFile strFile = new StrFile();
            strFile.setContent(pemStr);
            strFile.setFileName("fullchain.pem");
            strFile.setContentType("text/plain");
            R<SysFile> r = this.remoteFileService.uploadStrToMinio(strFile);
            if (r.getCode() != Constants.SUCCESS) {
               CompletableFuture.completedFuture((Object)null);
               return;
            }

            String pemUrl = ((SysFile)r.getData()).getUrl();
            sslLog.setPem(pemUrl);
            strFile = new StrFile();
            strFile.setContent(keyStr);
            strFile.setFileName("privkey.pem");
            strFile.setContentType("text/plain");
            r = this.remoteFileService.uploadStrToMinio(strFile);
            if (r.getCode() == Constants.SUCCESS) {
               String keyUrl = ((SysFile)r.getData()).getUrl();
               sslLog.setKey(keyUrl);
               sslLog.setCreateId(sslLog.getCreateId());
               sslLog.setCreateName(sslLog.getCreateName());
               sslLog.setCreateTime(new Date());
               this.snSslLogMapper.insertSnSslLog(sslLog);
               break label87;
            }

            CompletableFuture.completedFuture((Object)null);
         } catch (Exception var19) {
            Exception e = var19;
            log.error("缓存证书失败：" + e.getMessage());
            break label87;
         } finally {
            shell.disconnect();
         }

         return;
      }

      if (task.getTaskName().equals("部署基础环境ssl证书")) {
         SnServer up = new SnServer();
         up.setServerId(task.getServerId());
         up.setSslTime(sslLog.getExpireTime());
         up.setSslPem(sslLog.getPem());
         up.setSslKey(sslLog.getKey());
         this.snServerMapper.updateSnServer(up);
      } else if (task.getTaskName().startsWith("部署客户端域名ssl证书")) {
         SnServerTemplate up = new SnServerTemplate();
         up.setId(task.getServerTemplateId());
         up.setSslTime(sslLog.getExpireTime());
         up.setSslPem(sslLog.getPem());
         up.setSslKey(sslLog.getKey());
         this.snServerTemplateMapper.updateSnServerTemplate(up);
      }

      CompletableFuture.completedFuture((Object)null);
   }
}
